@@ -83,10 +83,16 @@ void arrayTy::prettyprint(ostream &out, Int indent)
 // NOTE: Can this be merged with trans somehow?
 void arrayTy::addOps(coenv &e, record *r)
 {
-  types::array *t=dynamic_cast<types::array *>(trans(e, true));
-  e.e.addArrayOps(t);
-  if (r)
-    r->e.addArrayOps(t);
+  types::ty *t=trans(e, true);
+
+  // Only add ops if it is an array (and not, say, an error)
+  if (t->kind == types::ty_array) {
+    types::array *at=dynamic_cast<types::array *>(t);
+    assert(at);
+    e.e.addArrayOps(at);
+    if (r)
+      r->e.addArrayOps(at);
+  }
 }
   
 types::ty *arrayTy::trans(coenv &e, bool tacit)
@@ -94,6 +100,10 @@ types::ty *arrayTy::trans(coenv &e, bool tacit)
   types::ty *ct = cell->trans(e, tacit);
   assert(ct);
 
+  // Don't make an array of errors.
+  if (ct->kind == types::ty_error)
+    return ct;
+
   types::array *t = dims->truetype(ct);
   assert(t);
   
@@ -118,7 +128,7 @@ types::ty *tyEntryTy::trans(coenv &, bool) {
 
 vm::lambda *runnable::transAsCodelet(coenv &e)
 {
-  coder c=e.c.newCodelet();
+  coder c=e.c.newCodelet(getPos());
   coenv ce(c, e.e);
   markTrans(ce);
   return c.close();
@@ -161,14 +171,14 @@ void block::transAsRecordBody(coenv &e, record *r)
   e.c.closeRecord();
 }
 
-record *block::transAsFile(genv& ge, symbol *id)
+record *block::transAsFile(genv& ge, symbol id)
 {
   // Create the new module.
-  record *r = new record(id, new frame(0,0));
+  record *r = new record(id, new frame(id,0,0));
 
   // Create coder and environment to translate the module.
   // File-level modules have dynamic fields by default.
-  coder c(r, 0);
+  coder c(getPos(), r, 0);
   env e(ge);
   coenv ce(c, e);
 
@@ -191,6 +201,28 @@ bool block::returns() {
       return true;
   return false;
 }
+
+vardec *block::asVardec()
+{
+  vardec *var = 0;
+  for (list<runnable *>::iterator p=stms.begin();
+       p != stms.end();
+       ++p)
+  {
+    vardec *v = dynamic_cast<vardec *>(*p);
+    if (v) {
+      if (var)
+        // Multiple vardecs.
+        return 0;
+      var = v;
+    }
+    else if (!dynamic_cast<emptyStm *>(*p))
+      // Failure due to another runnable in the block.
+      return 0;
+  }
+
+  return var;
+}
   
 
 void dec::prettyprint(ostream &out, Int indent)
@@ -304,7 +336,7 @@ void modifiedRunnable::transAsField(coenv &e, record *r)
 void decidstart::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "decidstart '" << *id << "'\n";
+  out << "decidstart '" << id << "'\n";
 
   if (dims)
     dims->prettyprint(out, indent+1);
@@ -336,7 +368,7 @@ void decidstart::addOps(types::ty *base, coenv &e, record *r)
 void fundecidstart::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "fundecidstart '" << *id << "'\n";
+  out << "fundecidstart '" << id << "'\n";
 
   if (dims)
     dims->prettyprint(out, indent+1);
@@ -404,11 +436,11 @@ varEntry *makeVarEntry(position pos, coenv &e, record *r, types::ty *t) {
 
 
 // Defined in constructor.cc.
-bool definesImplicitConstructor(coenv &e, record *r, varEntry *v, symbol *id);
+bool definesImplicitConstructor(coenv &e, record *r, varEntry *v, symbol id);
 void addConstructorFromInitializer(position pos, coenv &e, record *r,
                                    varEntry *init);
 
-void addVar(coenv &e, record *r, varEntry *v, symbol *id)
+void addVar(coenv &e, record *r, varEntry *v, symbol id)
 {
   // Test for 'operator init' definitions that implicitly define constructors:
   if (definesImplicitConstructor(e, r, v, id))
@@ -434,26 +466,55 @@ void initializeVar(position pos, coenv &e, varEntry *v, varinit *init)
   }
   
   v->getLocation()->encode(WRITE, pos, e.c);
-  e.c.encode(inst::pop);
+  e.c.encodePop();
+}
+
+types::ty *inferType(position pos, coenv &e, varinit *init)
+{
+  if (!init) {
+    em.error(pos);
+    em << "inferred variable declaration without initializer";
+    return primError();
+  }
+
+  exp *base = dynamic_cast<exp *>(init);
+  if (base) {
+    types::ty *t = base->cgetType(e);
+    if (t->kind != ty_overloaded)
+      return t;
+  }
+
+  em.error(pos);
+  em << "could not infer type of initializer";
+  return primError();
 }
 
 void createVar(position pos, coenv &e, record *r,
-               symbol *id, types::ty *t, varinit *init)
+               symbol id, types::ty *t, varinit *init)
 {
+  // I'm not sure how to handle inferred types in these cases.
+  assert(t->kind != types::ty_inferred);
+
   varEntry *v=makeVarEntry(pos, e, r, t);
   addVar(e, r, v, id);
   initializeVar(pos, e, v, init);
 }
 
 void createVarOutOfOrder(position pos, coenv &e, record *r,
-                         symbol *id, types::ty *t, varinit *init)
+                         symbol id, types::ty *t, varinit *init)
 {
+  /* For declarations such as "var x = 5;", infer the type from the
+   * initializer.
+   */
+  if (t->kind == types::ty_inferred)
+    t = inferType(pos, e, init);
+
   varEntry *v=makeVarEntry(pos, e, r, t);
   initializeVar(pos, e, v, init);
   addVar(e, r, v, id);
 }
 
-void addTypeWithPermission(coenv &e, record *r, tyEntry *base, symbol *id)
+void addTypeWithPermission(coenv &e, record *r, tyEntry *base, symbol id)
 {
   // Only bother encoding permissions for private types.
   tyEntry *ent = (r && e.c.getPermission()==PRIVATE) ?
@@ -531,6 +592,23 @@ void vardec::transAsTypedefField(coenv &e, record *r)
   decs->transAsTypedefField(e, base->transAsTyEntry(e, r), r);
 }
 
+symbol vardec::singleName()
+{
+  decid *did = decs->singleEntry();
+  if (!did)
+    return symbol::nullsym;
+  return did->getStart()->getName();
+}
+
+types::ty *vardec::singleGetType(coenv &e)
+{
+  decid *did = decs->singleEntry();
+  if (!did)
+    return 0;
+  return did->getStart()->getType(base->trans(e), e);
+}
+
+
 // Helper class for imports.  This essentially evaluates to the run::loadModule
 // function.  However, that function returns different types of records
 // depending on the filename given to it, so we cannot add it to the
@@ -544,6 +622,10 @@ public:
   loadModuleExp(position pos, record *imp)
     : exp(pos), imp(imp), ft(new function(imp,primString())) {}
 
+  void prettyprint(ostream &out, Int indent) {
+    prettyname(out, "loadModuleExp", indent);
+  }
+
   types::ty *trans(coenv &) {
     em.compiler(getPos());
     em << "trans called for loadModuleExp";
@@ -567,12 +649,12 @@ public:
 
 // Creates a local variable to hold the import and translate the accessing of
 // the import, but doesn't add the import to the environment.
-varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
+varEntry *accessModule(position pos, coenv &e, record *r, symbol id)
 {
-  record *imp=e.e.getModule(id, (string)*id);
+  record *imp=e.e.getModule(id, (string)id);
   if (!imp) {
     em.error(pos);
-    em << "could not load module '" << *id << "'";
+    em << "could not load module '" << id << "'";
     em.sync();
     return 0;
   }
@@ -580,7 +662,7 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
     // Create a varinit that evaluates to the module.
     // This is effectively the expression "loadModule(filename)".
     callExp init(pos, new loadModuleExp(pos, imp),
-                 new stringExp(pos, *id));
+                 new stringExp(pos, (string)id));
 
     // The varEntry should have whereDefined()==0 as it is not defined inside
     // the record r.
@@ -594,7 +676,7 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
 void idpair::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "idpair (" << "'" << *src << "' as " << *dest << ")\n";
+  out << "idpair (" << "'" << src << "' as " << dest << ")\n";
 }
 
 void idpair::transAsAccess(coenv &e, record *r)
@@ -615,7 +697,7 @@ void idpair::transAsUnravel(coenv &e, record *r,
     r->e.add(src, dest, source, qualifier, e.c);
   if (!e.e.add(src, dest, source, qualifier, e.c)) {
     em.error(getPos());
-    em << "no matching types or fields of name '" << *src << "'";
+    em << "no matching types or fields of name '" << src << "'";
   }
 }
 
@@ -698,7 +780,7 @@ fromdec::qualifier unraveldec::getQualifier(coenv &e, record *)
 void fromaccessdec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "fromaccessdec '" << *id << "'\n";
+  out << "fromaccessdec '" << id << "'\n";
   idpairlist *f=this->fields;
   if(f) f->prettyprint(out, indent+1);
 }
@@ -759,7 +841,7 @@ void typedec::prettyprint(ostream &out, Int indent)
 void recorddec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "structdec '" << *id << "'\n";
+  out << "structdec '" << id << "'\n";
 
   body->prettyprint(out, indent+1);
 }
@@ -798,7 +880,7 @@ void recorddec::transAsField(coenv &e, record *parent)
     parent->e.addRecordOps(r);
 
   // Start translating the initializer.
-  coder c=e.c.newRecordInit(r);
+  coder c=e.c.newRecordInit(getPos(), r);
   coenv re(c,e.e);
   
   body->transAsRecordBody(re, r);
