@@ -18,7 +18,7 @@ namespace camp {
 class drawLabel : public virtual drawElement {
 protected:
   string label,size;
-  transform T;		// A linear (shiftless) transformation.
+  transform T;          // A linear (shiftless) transformation.
   pair position;
   pair align;
   pair scale;
@@ -33,10 +33,10 @@ protected:
   
 public:
   drawLabel(string label, string size, transform T, pair position,
-	    pair align, pen pentype)
+            pair align, pen pentype)
     : label(label), size(size), T(shiftless(T)), position(position),
       align(align), pentype(pentype), width(0.0), height(0.0), depth(0.0),
-    havebounds(false), suppress(false), enabled(false) {} 
+      havebounds(false), suppress(false), enabled(false) {} 
   
   virtual ~drawLabel() {}
 
@@ -46,11 +46,6 @@ public:
     
   void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&);
   
-  bool texbounds(iopipestream& tex, string& s, const char **abort, bool warn);
-    
-  int wait(iopipestream &tex, const char *s, const char **abort,
-	   bool ignore=false);
-  
   bool islabel() {
     return true;
   }
@@ -68,12 +63,15 @@ private:
   pair shift;
 public:
   drawLabelPath(string label, string size, path src,
-		string justify, pair shift, pen pentype) : 
-    drawLabel(label,size,identity,pair(0.0,0.0),align,pentype),
+                string justify, pair shift, pen pentype) : 
+    drawLabel(label,size,identity,pair(0.0,0.0),pair(0.0,0.0),pentype),
     drawPathPenBase(src,pentype), justify(justify), shift(shift) {}
   
   virtual ~drawLabelPath() {}
 
+  bool svg() {return true;}
+  bool svgpng() {return true;}
+  
   void bounds(bbox& b, iopipestream& tex, boxvector&, bboxlist&);
   
   bool write(texfile *out, const bbox&);
@@ -81,6 +79,10 @@ public:
   drawElement *transformed(const transform& t);
 };
 
+void setpen(iopipestream& tex, const string& texengine, const pen& pentype);
+void texbounds(double& width, double& height, double& depth,
+               iopipestream& tex, string& s);
+
 }
 
 #endif
