@@ -331,6 +331,7 @@ public:
 };
 
 path3 transformed(const vm::array& t, const path3& p);
+path3 transformed(const double* t, const path3& p);
   
 extern path3 nullpath3;
 extern const unsigned maxdepth;
@@ -344,9 +345,9 @@ bool intersections(double& s, double& t, std::vector<double>& S,
 void intersections(std::vector<double>& S, path3& g,
                    const triple& p, const triple& q, double fuzz);
 
-void intersections(std::vector<double>& T, std::vector<double>& U,
+bool intersections(std::vector<double>& T, std::vector<double>& U,
                    std::vector<double>& V, path3& p, triple *P,
-                   double fuzz, unsigned depth=maxdepth);
+                   double fuzz, bool single, unsigned depth=maxdepth);
 bool intersections(double& U, double& V, const triple& v, triple *P,
                    double fuzz, unsigned depth=maxdepth);
 
@@ -361,27 +362,31 @@ inline triple displacement(const triple& z, const triple& p, const triple& q)
   triple Q=unit(q-p);
   return Z-dot(Z,Q)*Q;
 }
+
+typedef double bound_double(double *P, double (*m)(double, double), double b,
+                     double fuzz, int depth);
+
+typedef double bound_triple(triple *P, double (*m)(double, double),
+                       double (*f)(const triple&), double b, double fuzz,
+                       int depth);
   
-double xratio(const triple& v);
-double yratio(const triple& v);
+bound_double bound,boundtri;
 
 double bound(triple z0, triple c0, triple c1, triple z1,
              double (*m)(double, double),
              double (*f)(const triple&),
              double b, double fuzz, int depth=maxdepth);
 double bound(double *p, double (*m)(double, double),
-             double b, double fuzz, int depth=maxdepth);
+             double b, double fuzz, int depth);
 double bound(triple *P, double (*m)(double, double),
              double (*f)(const triple&), double b, double fuzz,
-             int depth=maxdepth);
-
-inline void store(Triple& control, const triple& v)
-{
-  control[0]=v.getx();
-  control[1]=v.gety();
-  control[2]=v.getz();
-}
+             int depth);
 
+double boundtri(double *P, double (*m)(double, double), double b,
+                double fuzz, int depth);
+double boundtri(triple *P, double (*m)(double, double),
+                double (*f)(const triple&), double b, double fuzz,
+                int depth);
 }
 
 #ifndef BROKEN_COMPILER
