@@ -11,6 +11,13 @@
 #include <sys/types.h>
 #include <iostream>
 #include <cstdlib>
+
+#ifdef __CYGWIN__
+extern "C" int sigaddset(sigset_t *set, int signum);
+extern "C" int sigemptyset(sigset_t *set); 
+extern "C" int sigaction(int signum, const struct sigaction *act, struct sigaction *oldact);
+#endif
+
 #include <csignal>
 
 #include "common.h"
@@ -101,6 +108,7 @@ extern "C" double yn(Int n, double x);
 extern "C" int snprintf(char *str, size_t size, const char *format,...);
 extern "C" int isnan(double);
 #include <stdio.h>
+extern "C" FILE *fdopen(int fd, const char *mode);
 extern "C" int fileno(FILE *);
 extern "C" char *strptime(const char *s, const char *format, struct tm *tm);
 extern "C" int setenv(const char *name, const char *value, int overwrite);
